@@ -29,6 +29,7 @@ import (
 	"github.com/blang/semver"
 	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	"github.com/coreos/prometheus-operator/pkg/k8sutil"
+	"github.com/coreos/prometheus-operator/pkg/operator"
 	"github.com/pkg/errors"
 )
 
@@ -131,14 +132,18 @@ func makeStatefulSet(am *monitoringv1.Alertmanager, old *appsv1.StatefulSet, con
 			},
 		})
 	} else {
-		pvcTemplate := storageSpec.VolumeClaimTemplate
+		pvcTemplate := operator.MakeVolumeClaimTemplate(storageSpec.VolumeClaimTemplate)
 		if pvcTemplate.Name == "" {
 			pvcTemplate.Name = volumeName(am.Name)
 		}
-		pvcTemplate.Spec.AccessModes = []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce}
+		if storageSpec.VolumeClaimTemplate.Spec.AccessModes == nil {
+			pvcTemplate.Spec.AccessModes = []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce}
+		} else {
+			pvcTemplate.Spec.AccessModes = storageSpec.VolumeClaimTemplate.Spec.AccessModes
+		}
 		pvcTemplate.Spec.Resources = storageSpec.VolumeClaimTemplate.Spec.Resources
 		pvcTemplate.Spec.Selector = storageSpec.VolumeClaimTemplate.Spec.Selector
-		statefulset.Spec.VolumeClaimTemplates = append(statefulset.Spec.VolumeClaimTemplates, pvcTemplate)
+		statefulset.Spec.VolumeClaimTemplates = append(statefulset.Spec.VolumeClaimTemplates, *pvcTemplate)
 	}
 
 	if old != nil {
@@ -183,13 +188,13 @@ func makeStatefulSetService(p *monitoringv1.Alertmanager, config Config) *v1.Ser
 					Protocol:   v1.ProtocolTCP,
 				},
 				{
-					Name:       "mesh-tcp",
+					Name:       "tcp-mesh",
 					Port:       9094,
 					TargetPort: intstr.FromInt(9094),
 					Protocol:   v1.ProtocolTCP,
 				},
 				{
-					Name:       "mesh-udp",
+					Name:       "udp-mesh",
 					Port:       9094,
 					TargetPort: intstr.FromInt(9094),
 					Protocol:   v1.ProtocolUDP,
@@ -463,53 +468,7 @@ func makeStatefulSetSpec(a *monitoringv1.Alertmanager, config Config) (*appsv1.S
 	terminationGracePeriod := int64(120)
 	finalLabels := config.Labels.Merge(podLabels)
 
-	defaultContainers := []v1.Container{
-		{
-			Args:           amArgs,
-			Name:           "alertmanager",
-			Image:          image,
-			Ports:          ports,
-			VolumeMounts:   amVolumeMounts,
-			LivenessProbe:  livenessProbe,
-			ReadinessProbe: readinessProbe,
-			Resources:      a.Spec.Resources,
-			Env: []v1.EnvVar{
-				{
-					// Necessary for '--cluster.listen-address' flag
-					Name: "POD_IP",
-					ValueFrom: &v1.EnvVarSource{
-						FieldRef: &v1.ObjectFieldSelector{
-							FieldPath: "status.podIP",
-						},
-					},
-				},
-			},
-			TerminationMessagePolicy: v1.TerminationMessageFallbackToLogsOnError,
-		}, {
-			Name:  "config-reloader",
-			Image: config.ConfigReloaderImage,
-			Args: []string{
-				fmt.Sprintf("-webhook-url=%s", localReloadURL),
-				fmt.Sprintf("-volume-dir=%s", alertmanagerConfDir),
-			},
-			VolumeMounts: []v1.VolumeMount{
-				{
-					Name:      "config-volume",
-					ReadOnly:  true,
-					MountPath: alertmanagerConfDir,
-				},
-			},
-			Resources:                resources,
-			TerminationMessagePolicy: v1.TerminationMessageFallbackToLogsOnError,
-		},
-	}
-
-	containers, err := k8sutil.MergePatchContainers(defaultContainers, a.Spec.Containers)
-	if err != nil {
-		return nil, errors.Wrap(err, "failed to merge containers spec")
-	}
-
-	// PodManagementPolicy is set to Parallel to mitigate issues in kubernetes: https://github.com/kubernetes/kubernetes/issues/60164
+	// PodManagementPolicy is set to Parallel to mitigate issues in kuberentes: https://github.com/kubernetes/kubernetes/issues/60164
 	// This is also mentioned as one of limitations of StatefulSets: https://kubernetes.io/docs/concepts/workloads/controllers/statefulset/#limitations
 	return &appsv1.StatefulSetSpec{
 		ServiceName:         governingServiceName,
@@ -531,12 +490,51 @@ func makeStatefulSetSpec(a *monitoringv1.Alertmanager, config Config) (*appsv1.S
 				PriorityClassName:             a.Spec.PriorityClassName,
 				TerminationGracePeriodSeconds: &terminationGracePeriod,
 				InitContainers:                a.Spec.InitContainers,
-				Containers:                    containers,
-				Volumes:                       volumes,
-				ServiceAccountName:            a.Spec.ServiceAccountName,
-				SecurityContext:               securityContext,
-				Tolerations:                   a.Spec.Tolerations,
-				Affinity:                      a.Spec.Affinity,
+				Containers: append([]v1.Container{
+					{
+						Args:           amArgs,
+						Name:           "alertmanager",
+						Image:          image,
+						Ports:          ports,
+						VolumeMounts:   amVolumeMounts,
+						LivenessProbe:  livenessProbe,
+						ReadinessProbe: readinessProbe,
+						Resources:      a.Spec.Resources,
+						Env: []v1.EnvVar{
+							{
+								// Necessary for '--cluster.listen-address' flag
+								Name: "POD_IP",
+								ValueFrom: &v1.EnvVarSource{
+									FieldRef: &v1.ObjectFieldSelector{
+										FieldPath: "status.podIP",
+									},
+								},
+							},
+						},
+						TerminationMessagePolicy: v1.TerminationMessageFallbackToLogsOnError,
+					}, {
+						Name:  "config-reloader",
+						Image: config.ConfigReloaderImage,
+						Args: []string{
+							fmt.Sprintf("-webhook-url=%s", localReloadURL),
+							fmt.Sprintf("-volume-dir=%s", alertmanagerConfDir),
+						},
+						VolumeMounts: []v1.VolumeMount{
+							{
+								Name:      "config-volume",
+								ReadOnly:  true,
+								MountPath: alertmanagerConfDir,
+							},
+						},
+						Resources:                resources,
+						TerminationMessagePolicy: v1.TerminationMessageFallbackToLogsOnError,
+					},
+				}, a.Spec.Containers...),
+				Volumes:            volumes,
+				ServiceAccountName: a.Spec.ServiceAccountName,
+				SecurityContext:    securityContext,
+				Tolerations:        a.Spec.Tolerations,
+				Affinity:           a.Spec.Affinity,
 			},
 		},
 	}, nil
